package com.jiuqi.util.comparator;

import java.util.ArrayList;
import java.util.List;
import java.util.Comparator;

public final class AggreatedComparator implements Comparator
{
    private List comparators;
    
    public AggreatedComparator() {
        this.comparators = new ArrayList();
    }
    
    public int compare(final Object o1, final Object o2) {
        if (o1 == o2) {
            return 0;
        }
        if (o1 == null) {
            return -1;
        }
        if (o2 == null) {
            return 1;
        }
        final Object[] arr1 = (Object[])o1;
        final Object[] arr2 = (Object[])o2;
        for (int len = Math.min(Math.min(arr1.length, arr2.length), this.comparators.size()), i = 0; i < len; ++i) {
            final Comparator c = this.comparators.get(i);
            final int ret = c.compare(arr1[i], arr2[i]);
            if (ret != 0) {
                return ret;
            }
        }
        return 0;
    }
    
    @Override
    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (this == null || obj == null) {
            return false;
        }
        if (!(obj instanceof AggreatedComparator)) {
            return false;
        }
        final AggreatedComparator cmp = (AggreatedComparator)obj;
        if (this.comparators.size() != cmp.comparators.size()) {
            return false;
        }
        for (int i = 0; i < this.comparators.size(); ++i) {
            if (!this.comparators.get(i).equals(cmp.comparators.get(i))) {
                return false;
            }
        }
        return true;
    }
    
    @Override
    public String toString() {
        final StringBuffer result = new StringBuffer("[聚合比较器");
        for (int i = 0; i < this.comparators.size(); ++i) {
            result.append(", ");
            result.append(this.comparators.get(i));
        }
        result.append("]");
        return result.toString();
    }
    
    public void addComparator(final Comparator comparator) {
        this.comparators.add(comparator);
    }
    
    public void removeComparator(final Comparator comparator) {
        this.comparators.remove(comparator);
    }
    
    public Comparator[] getAllComparators() {
        return this.comparators.toArray(new Comparator[0]);
    }
}
